==> pympipool-0.8.4.tar.gz: pympipool-0.8.4/tests/test_flux_executor.py <==
from concurrent.futures import Future
import os
from queue import Queue
import unittest

import numpy as np

from pympipool.shared.executorbase import cloudpickle_register, execute_parallel_tasks


try:
    import flux.job
    from pympipool.scheduler.flux import (
        PyFluxExecutor,
        FluxPythonInterface,
    )

    skip_flux_test = "FLUX_URI" not in os.environ
    pmi = os.environ.get("PYMPIPOOL_PMIX", None)
except ImportError:
    skip_flux_test = True


def calc(i):
    return i


def mpi_funct(i):
    from mpi4py import MPI

    size = MPI.COMM_WORLD.Get_size()
    rank = MPI.COMM_WORLD.Get_rank()
    return i, size, rank


def get_global(memory=None):
    return memory


def set_global():
    return {"memory": np.array([5])}


@unittest.skipIf(
    skip_flux_test, "Flux is not installed, so the flux tests are skipped."
)
class TestFlux(unittest.TestCase):
    def setUp(self):
        self.executor = flux.job.FluxExecutor()

    def test_flux_executor_serial(self):
        with PyFluxExecutor(max_workers=2, executor=self.executor) as exe:
            fs_1 = exe.submit(calc, 1)
            fs_2 = exe.submit(calc, 2)
            self.assertEqual(fs_1.result(), 1)
            self.assertEqual(fs_2.result(), 2)
            self.assertTrue(fs_1.done())
            self.assertTrue(fs_2.done())

    def test_flux_executor_threads(self):
        with PyFluxExecutor(
            max_workers=1, threads_per_core=2, executor=self.executor
        ) as exe:
            fs_1 = exe.submit(calc, 1)
            fs_2 = exe.submit(calc, 2)
            self.assertEqual(fs_1.result(), 1)
            self.assertEqual(fs_2.result(), 2)
            self.assertTrue(fs_1.done())
            self.assertTrue(fs_2.done())

    def test_flux_executor_parallel(self):
        with PyFluxExecutor(
            max_workers=1, cores_per_worker=2, executor=self.executor, pmi=pmi
        ) as exe:
            fs_1 = exe.submit(mpi_funct, 1)
            self.assertEqual(fs_1.result(), [(1, 2, 0), (1, 2, 1)])
            self.assertTrue(fs_1.done())

    def test_single_task(self):
        with PyFluxExecutor(
            max_workers=1, cores_per_worker=2, executor=self.executor, pmi=pmi
        ) as p:
            output = p.map(mpi_funct, [1, 2, 3])
        self.assertEqual(
            list(output),
            [[(1, 2, 0), (1, 2, 1)], [(2, 2, 0), (2, 2, 1)], [(3, 2, 0), (3, 2, 1)]],
        )

    def test_execute_task(self):
        f = Future()
        q = Queue()
        q.put({"fn": calc, "args": (), "kwargs": {"i": 2}, "future": f})
        q.put({"shutdown": True, "wait": True})
        cloudpickle_register(ind=1)
        execute_parallel_tasks(
            future_queue=q,
            cores=1,
            executor=self.executor,
            interface_class=FluxPythonInterface,
        )
        self.assertEqual(f.result(), 2)
        q.join()

    def test_execute_task_threads(self):
        f = Future()
        q = Queue()
        q.put({"fn": calc, "args": (), "kwargs": {"i": 2}, "future": f})
        q.put({"shutdown": True, "wait": True})
        cloudpickle_register(ind=1)
        execute_parallel_tasks(
            future_queue=q,
            cores=1,
            threads_per_core=1,
            executor=self.executor,
            interface_class=FluxPythonInterface,
        )
        self.assertEqual(f.result(), 2)
        q.join()

    def test_internal_memory(self):
        with PyFluxExecutor(
            max_workers=1,
            cores_per_worker=1,
            init_function=set_global,
            executor=self.executor,
        ) as p:
            f = p.submit(get_global)
            self.assertFalse(f.done())
            self.assertEqual(f.result(), np.array([5]))
            self.assertTrue(f.done())

==> pympipool-0.8.5.tar.gz: pympipool-0.8.5/tests/test_flux_executor.py <==
from concurrent.futures import Future
import os
from queue import Queue
import unittest

import numpy as np

from pympipool.interactive.executor import InteractiveExecutor
from pympipool.shared.executor import cloudpickle_register, execute_parallel_tasks


try:
    import flux.job
    from pympipool.interactive.flux import FluxPythonInterface

    skip_flux_test = "FLUX_URI" not in os.environ
    pmi = os.environ.get("PYMPIPOOL_PMIX", None)
except ImportError:
    skip_flux_test = True


def calc(i):
    return i


def mpi_funct(i):
    from mpi4py import MPI

    size = MPI.COMM_WORLD.Get_size()
    rank = MPI.COMM_WORLD.Get_rank()
    return i, size, rank


def get_global(memory=None):
    return memory


def set_global():
    return {"memory": np.array([5])}


@unittest.skipIf(
    skip_flux_test, "Flux is not installed, so the flux tests are skipped."
)
class TestFlux(unittest.TestCase):
    def setUp(self):
        self.executor = flux.job.FluxExecutor()

    def test_flux_executor_serial(self):
        with InteractiveExecutor(
            max_workers=2,
            executor_kwargs={"executor": self.executor},
            interface_class=FluxPythonInterface,
        ) as exe:
            fs_1 = exe.submit(calc, 1)
            fs_2 = exe.submit(calc, 2)
            self.assertEqual(fs_1.result(), 1)
            self.assertEqual(fs_2.result(), 2)
            self.assertTrue(fs_1.done())
            self.assertTrue(fs_2.done())

    def test_flux_executor_threads(self):
        with InteractiveExecutor(
            max_workers=1,
            executor_kwargs={"executor": self.executor, "threads_per_core": 2},
            interface_class=FluxPythonInterface,
        ) as exe:
            fs_1 = exe.submit(calc, 1)
            fs_2 = exe.submit(calc, 2)
            self.assertEqual(fs_1.result(), 1)
            self.assertEqual(fs_2.result(), 2)
            self.assertTrue(fs_1.done())
            self.assertTrue(fs_2.done())

    def test_flux_executor_parallel(self):
        with InteractiveExecutor(
            max_workers=1,
            executor_kwargs={"executor": self.executor, "cores": 2, "pmi": pmi},
            interface_class=FluxPythonInterface,
        ) as exe:
            fs_1 = exe.submit(mpi_funct, 1)
            self.assertEqual(fs_1.result(), [(1, 2, 0), (1, 2, 1)])
            self.assertTrue(fs_1.done())

    def test_single_task(self):
        with InteractiveExecutor(
            max_workers=1,
            executor_kwargs={"executor": self.executor, "cores": 2, "pmi": pmi},
            interface_class=FluxPythonInterface,
        ) as p:
            output = p.map(mpi_funct, [1, 2, 3])
        self.assertEqual(
            list(output),
            [[(1, 2, 0), (1, 2, 1)], [(2, 2, 0), (2, 2, 1)], [(3, 2, 0), (3, 2, 1)]],
        )

    def test_execute_task(self):
        f = Future()
        q = Queue()
        q.put({"fn": calc, "args": (), "kwargs": {"i": 2}, "future": f})
        q.put({"shutdown": True, "wait": True})
        cloudpickle_register(ind=1)
        execute_parallel_tasks(
            future_queue=q,
            cores=1,
            executor=self.executor,
            interface_class=FluxPythonInterface,
        )
        self.assertEqual(f.result(), 2)
        q.join()

    def test_execute_task_threads(self):
        f = Future()
        q = Queue()
        q.put({"fn": calc, "args": (), "kwargs": {"i": 2}, "future": f})
        q.put({"shutdown": True, "wait": True})
        cloudpickle_register(ind=1)
        execute_parallel_tasks(
            future_queue=q,
            cores=1,
            threads_per_core=1,
            executor=self.executor,
            interface_class=FluxPythonInterface,
        )
        self.assertEqual(f.result(), 2)
        q.join()

    def test_internal_memory(self):
        with InteractiveExecutor(
            max_workers=1,
            executor_kwargs={
                "executor": self.executor,
                "cores": 1,
                "init_function": set_global,
            },
            interface_class=FluxPythonInterface,
        ) as p:
            f = p.submit(get_global)
            self.assertFalse(f.done())
            self.assertEqual(f.result(), np.array([5]))
            self.assertTrue(f.done())

    def test_interface_exception(self):
        with self.assertRaises(ValueError):
            flux_interface = FluxPythonInterface(
                executor=self.executor, oversubscribe=True
            )
            flux_interface.bootup(command_lst=[])
        with self.assertRaises(ValueError):
            flux_interface = FluxPythonInterface(executor=self.executor)
            flux_interface.bootup(command_lst=[], prefix_path="/path/to/conda/env")
        with self.assertRaises(ValueError):
            flux_interface = FluxPythonInterface(executor=self.executor)
            flux_interface.bootup(command_lst=[], prefix_name="conda_env_name")
